/**
 * Utility for generating JWTs 
 */

import {Encoding, HashAlg} from 'N/crypto'
import * as encode from 'N/encode'
import * as https from 'N/https'

/**
 * returns the given data base64 encoded with padding removed.
 * @param data data to encode to base64
 */
function base64encodeNoPadding(data: string) {
  return encode.convert({
    string: data,
    inputEncoding: encode.Encoding.UTF_8,
    outputEncoding: Encoding.BASE_64_URL_SAFE
  }).replace(/=/g, "")
}

/**
 * returns  JWT header as an object
 */
function jwtHeader() {
  return {
    alg: 'HS512' //  API expects HMAC SHA512
  }
}

/**
 * returns  API JWT payload as object. The only dynamic property is the user id - all other properties
 * are constants or calculated parts of the JWT payload.
 */
function jwtPayload(userid: number) {
  const now = Math.floor(Date.now()/1000)
  return {
    exp: now + 300, // expire 5 minutes from now,
    iat: now,
    sub: "access_token",
    user_id: userid
  }
}

/**
 * Computes and returns a JWT suitable for use on API 
 * @param secretid netsuite API secret id to use in HMAC signature
 */
export function makeJWT(userid: number, secretid: string) {
  const base64header = base64encodeNoPadding(JSON.stringify(jwtHeader()))
  const base64payload = base64encodeNoPadding(JSON.stringify(jwtPayload(userid)))
  const key = https.createSecretKey({
    secret: secretid,
    encoding: encode.Encoding.BASE_64
  })
  // for JWT, we sign the header and payload concatenated with a dot (.) between parts
  const signature = https.createSecureString({
    input: `${base64header}.${base64payload}`
  }).hmac({
    algorithm: HashAlg.SHA512,
    key: key,
    resultEncoding: encode.Encoding.BASE_64_URL_SAFE
  }).replaceString({pattern: '=', replacement: ''})
  // token consists of header.payload.signature
  return https.createSecureString({
    input: `${base64header}.${base64payload}.`
  }).appendSecureString({secureString:signature, keepEncoding:true})
}



